#include <iostream>
#include <typeinfo>
#include <sstream>
#include <iomanip>
#include <vector>

namespace Color {
    enum Code {
        FG_RED      = 31,
        FG_GREEN    = 32,
        FG_YELLOW   = 33,
        FG_BLUE     = 34,
        FG_DEFAULT  = 39,
        BG_RED      = 41,
        BG_GREEN    = 42,
        BG_BLUE     = 44,
        BG_DEFAULT  = 49
    };
    class Modifier {
        Code code;
    public:
        Modifier (Code in_code) : code(in_code) {}
        friend std::ostream&
        operator<<(std::ostream& os, const Modifier& mod) {
            return os << "\033[" << mod.code << "m";
        }
    };
}

Color::Modifier red (Color::FG_RED);
Color::Modifier green (Color::FG_GREEN);
Color::Modifier yellow (Color::FG_YELLOW);
Color::Modifier def (Color::FG_DEFAULT);

class Zviratko {
protected:
    int zdravi, kalorie;
    std::vector<std::string> names;
public:
    Zviratko (int in_kalorie = 1000): kalorie(in_kalorie), zdravi(100) {}
    virtual ~Zviratko() {}
    std::string info () {
        std::stringstream ss;
        ss << green << std::setw(10) << "info " << def << std::left << std::setw(10) << this->name();
        ss << "| kalorie: " << std::setw(6) << this->kalorie;
        ss << "| zdravi: " << std::setw(4) << this->zdravi;
        return ss.str(); 
    }
    virtual void konzumace (Zviratko &z) {
        std::cout << red << std::setw(10) << "incident " << def; 
        std::cout << this->name() << " se pokousi sezrat " << z.name(1) << "\n";
        if (std::addressof(z) == std::addressof(*this)) {
            std::cout << yellow << std::setw(10) << "fail " << def << "zviratko nemuze sezrat samo sebe\n";
            return;
        }
        if (typeid(*this) == typeid(z)) {
            std::cout << yellow << std::setw(10) << "fail " << def << "kanibalismus mezi zviratky netolerujeme\n";
            return;
        }
        this->kalorie += z.kalorie;
    }
    virtual void konzumace (Zviratko *z) {
        std::cout << red << std::setw(10) << "incident " << def;
        std::cout << this->name() << " se pokousi sezrat " << z->name(1) << "\n";
        if (std::addressof(*z) == std::addressof(*this)) {
            std::cout << yellow << std::setw(10) << "fail " << def << "zviratko nemuze sezrat samo sebe\n";
            return;
        } 
        if (typeid(*this) == typeid(*z)) {
            std::cout << yellow << std::setw(10) << "fail " << def << "kanibalismus mezi zviratky netolerujeme\n";
            return;
        }
        this->kalorie += z->kalorie;
    }
    std::string name(int index = 0) {
        if (index > names.size()) 
            return typeid(*this).name();
        return names.at(index);
    } 
};

void operator<< (std::ostream &s, Zviratko &z) {
    s << z.info() << std::endl;
}

void operator<< (std::ostream &s, Zviratko *z) {
    s << z->info() << std::endl;
}

class Mys: public Zviratko {
public:
    Mys (): Zviratko (100) {
        names.push_back("mys");
        names.push_back("mys");
    }
};

class Kocka: public Zviratko {
public:
    Kocka (): Zviratko () {
        names.push_back ("kocka");
        names.push_back ("kocku");
    }
};


int main() {
    Kocka *k1 = new Kocka;
    Kocka *k2 = new Kocka;
    Kocka k3;
    Mys *m = new Mys;
    Mys m1;
    std::cout << m;
    std::cout << k1;
    k1->konzumace(m);
    m->konzumace(k1);
    k1->konzumace(k2);
    k3.konzumace (k3);
    m1.konzumace (m);
}